@@ -1,6 +1,6 @@
 /*
  * cwdaemon - morse sounding daemon for the parallel or serial port
- * Copyright (C) 2002 -2003 Joop Stakenborg <pg4i@amsat.org>
+ * Copyright (C) 2002 -2004 Joop Stakenborg <pg4i@amsat.org>
  *                       and many authors, see the AUTHORS file.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -84,15 +84,23 @@
 #if HAVE_SYS_RESOURCE_H
 # include <sys/resource.h>
 #endif
+#if (defined(__unix__) || defined(unix)) && !defined(USG)
+# include <sys/param.h>
+#endif
 
 #include "cwlib.h"
 #include "cwdaemon.h"
 
+#ifdef HAVE_DEV_PPBUS_PPI_H
+static char *actual_device(const char *in);
+#endif
+
 /* network vars */
-int sin_len;
+int sin_len, reply_socklen;
 int socket_descriptor;
-struct sockaddr_in k_sin;
+struct sockaddr_in k_sin, reply_sin;
 int port = 6789;		/* default UDP port we listen on */
+char reply_data[256];
 
 /* morse defaults */
 int morse_speed = 24;		/* speed (wpm) */
@@ -115,7 +123,7 @@ int aborting = 0;
 int sendingmorse = 0;
 
 struct timeval now, end, left;	/* PTT timers */
-struct timespec sleeptime, remainder; /* delay timers */
+struct timespec sleeptime, time_remainder; /* delay timers */
 
 #define MAXMORSE 4000
 char morsetext[MAXMORSE];
@@ -131,7 +139,7 @@ cwdevice cwdevice_ttys = {
 	footswitch:NULL
 };
 
-#ifdef HAVE_LINUX_PPDEV_H
+#if defined (HAVE_LINUX_PPDEV_H) || defined (HAVE_DEV_PPBUS_PPI_H)
 cwdevice cwdevice_lp = {
 	init:lp_init,
 	free:lp_free,
@@ -198,10 +206,10 @@ udelay (unsigned long us)
 	sleeptime.tv_sec = 0;
 	sleeptime.tv_nsec = us * 1000;
 
-	if (nanosleep (&sleeptime, &remainder) == -1)
+	if (nanosleep (&sleeptime, &time_remainder) == -1)
 	{
 		if (errno == EINTR)
-			nanosleep (&remainder, NULL);
+			nanosleep (&time_remainder, NULL);
 		else
 			errmsg ("Nanosleep");
 	}
@@ -403,6 +411,10 @@ recv_code (void)
 				{
 					cwdev->free (cwdev);
 					cwdev = &cwdevice_ttys;
+#ifdef BSD
+					if (!strncmp (message + 2, "ttyS", 4))
+						message[3] = 'd';
+#endif
 					cwdev->desc = message + 2;
 					cwdev->init (cwdev);
 				}
@@ -414,6 +426,22 @@ recv_code (void)
 					cwdev->desc = message + 2;
 					cwdev->init (cwdev);
 				}
+#endif
+#ifdef HAVE_DEV_PPBUS_PPI_H
+				else if (strncmp (message + 2, "parport", 7) == 0)
+				{
+					cwdev->free (cwdev);
+					cwdev = &cwdevice_lp;
+					cwdev->desc = actual_device(message + 2);
+					cwdev->init (cwdev);
+				}
+				else if (strncmp (message + 2, "ppi", 3) == 0)
+				{
+					cwdev->free (cwdev);
+					cwdev = &cwdevice_lp;
+					cwdev->desc = message + 2;
+					cwdev->init (cwdev);
+				}
 #endif
 				else debug ("Unknown device");
 				break;
@@ -513,6 +541,14 @@ recv_code (void)
 					cw_set_volume (morse_volume);
 				}
 				break;
+            case 'h':   /* send echo to main program when CW playing is done */
+                memcpy(&reply_sin, &k_sin, sizeof(reply_sin)); /* remember sender */
+                reply_socklen = sin_len;
+                strncpy(reply_data+1, message, sizeof(reply_data) - 2);
+                reply_data[sizeof(reply_data) - 1] = '\0';
+                reply_data[0]='h';
+                if (strlen (message) + 1 <= MAXMORSE - 1) strcat (morsetext, "^");
+                break;
 			}
 			return 0;
 		}
@@ -525,6 +561,7 @@ static void
 playmorsestring (char *x)
 {
 	int i = 0, validchar = 0;
+	char c;
 
 	sendingmorse = 1;
 	while (*x)
@@ -534,7 +571,7 @@ playmorsestring (char *x)
 			aborting = 0;
 			break;
 		}
-		char c = *x;
+		c = *x;
 		if ((c == '+') || (c == '-'))
 		{		/* speed in- & decrease */
 			if ((c == '+') && (morse_speed <= 58))
@@ -545,6 +582,15 @@ playmorsestring (char *x)
 		}
 		else if (c == '~')
 			cw_set_gap (2); /* 2 dots time additional for the next char */
+        else if (c == '^'){
+            debug("Echo '%s'", reply_data);
+            if (strlen(reply_data)==0) return;
+            sendto(socket_descriptor, reply_data, strlen(reply_data), 0, 
+                (struct sockaddr *)&reply_sin, reply_socklen);
+            strcpy(reply_data,"");
+            break;
+  
+        }
 		else
 		{
 			validchar = 1;
@@ -622,10 +668,16 @@ parsecommandline (int argc, char *argv[])
 			printf ("Usage: %s [option]...\n", PACKAGE);
 			printf ("       -d <device>   ");
 			printf ("Use a different device\n                     ");
-#ifdef HAVE_LINUX_PPDEV_H
+#if defined (HAVE_LINUX_PPDEV_H)
 			printf ("(e.g. ttyS0,1,2, parport0,1, etc. default = parport0)\n");
+#elif defined (HAVE_DEV_PPBUS_PPI_H)
+			printf ("(e.g. ttyd0,1,2, ppi0,1, etc. default = ppi0)\n");
+#else
+#ifdef BSD
+			printf ("(e.g. ttyd0,1,2, etc. default = ttyd0)\n");
 #else
 			printf ("(e.g. ttyS0,1,2, etc. default = ttyS0)\n");
+#endif
 #endif
 			printf ("       -h            ");
 			printf ("Display this help and exit\n");
@@ -659,19 +711,37 @@ parsecommandline (int argc, char *argv[])
 			else if (!strncmp (optarg, "parport", 7)
 				 && strlen (optarg) < 9)
 				cwdev = &cwdevice_lp;
+#endif
+#ifdef HAVE_DEV_PPBUS_PPI_H
+			else if ((strncmp (optarg, "parport", 7) == 0)
+				 && strlen (optarg) < 9)
+				cwdev = &cwdevice_lp;
+			else if ((strncmp (optarg, "ppi", 3) == 0)
+				 && strlen (optarg) < 5)
+				cwdev = &cwdevice_lp;
 #endif
 			else
 			{
 				printf ("%s: Wrong device %s, ", PACKAGE,
 					optarg);
-#ifdef HAVE_LINUX_PPDEV_H
+#if defined (HAVE_LINUX_PPDEV_H)
 				printf ("use one of ttyS0,1,2,3 or parport0,1 etc.\n");
+#elif defined (HAVE_DEV_PPBUS_PPI_H)
+				printf ("use one of ttyd0,1,2,3 or ppi0,1 etc.\n");
+#else
+#ifdef BSD
+				printf ("use one of ttyd0,1,2,3 etc.\n");
 #else
 				printf ("use one of ttyS0,1,2,3 etc.\n");
+#endif
 #endif
 				exit (1);
 			}
+#ifdef HAVE_DEV_PPBUS_PPI_H
+			cwdev->desc = actual_device(optarg);
+#else
 			cwdev->desc = optarg;
+#endif
 			break;
 		case 'n':
 			forking = 0;
@@ -774,13 +844,18 @@ main (int argc, char *argv[])
 	int bind_rc, close_rc;
 	long save_file_flags;
 
-#ifdef HAVE_LINUX_PPDEV_H
+#if defined HAVE_LINUX_PPDEV_H
 	cwdev->desc = "parport0";
+#elif defined HAVE_DEV_PPBUS_PPI_H
+	cwdev->desc = "ppi0";
 #else
+ #ifdef BSD
+	cwdev->desc = "ttyd0";
+ #else
 	cwdev->desc = "ttyS0";
+ #endif
 #endif
 
-
 	parsecommandline (argc, argv);
 	if (geteuid () != 0)
 	{
@@ -902,3 +977,32 @@ main (int argc, char *argv[])
 	}
 	exit (0);
 }
+
+#ifdef HAVE_DEV_PPBUS_PPI_H
+/*
+ * inputs	- parport linux name
+ * output	- pointer to *static* char with BSD equivalent name
+ * side effects - none
+ */
+
+static char *
+actual_device(const char *in)
+{
+	static char actual_name[MAX_DEVICE];
+	/*
+	 * parportN
+	 * 01234567
+	 */
+	if (strncmp(in, "parport", 7) == 0)
+	{
+		/* Note no possible overflow here, snprintf not needed. */
+		sprintf(actual_name, "ppi%c", in[7]);
+	}
+	else
+	{
+		strncpy(actual_name, in, MAX_DEVICE-2);
+		actual_name[MAX_DEVICE-1] = '\0';
+	}
+	return(actual_name);
+}
+#endif
