@@ -1,6 +1,6 @@
 /*
  * cwdaemon - morse sounding daemon for the parallel or serial port
- * Copyright (C) 2002 -2003 Joop Stakenborg <pg4i@amsat.org>
+ * Copyright (C) 2002 -2004 Joop Stakenborg <pg4i@amsat.org>
  *                       and many authors, see the AUTHORS file.
  *
  * This program is free oftware; you can redistribute it and/or modify
@@ -30,6 +30,8 @@
 #define MICROPHONE 0
 #define SOUNDCARD 1
 
+#define MAX_DEVICE 20
+
 typedef struct cwdev_s
 {
 	int (*init) (struct cwdev_s *);
@@ -48,7 +50,7 @@ cwdevice;
 void errmsg (char *info, ...);
 void debug (char *info, ...);
 
-#ifdef HAVE_LINUX_PPDEV_H
+#if defined (HAVE_LINUX_PPDEV_H) || defined (HAVE_DEV_PPBUS_PPI_H)
 int lp_init (cwdevice * dev);
 int lp_free (cwdevice * dev);
 int lp_reset (cwdevice * dev);
